#include "util.h"



phys_para load_para()
{
	phys_para para = 
		{   50,    			// size of the system
			1.0,		// hopping constant t
			3.0,		// d*g coupling constant V
			0.0,       // chemical potential   es
			1,0.0,0.0,0.0			// hopping stride of d
	 	};
	 	


	FILE * f = fopen("parameters.txt","r");
	if(f!=NULL)
	{
		fscanf(f, 
		"L=%d\n t=%g\n V=%g\n es=%g\n stride=%d\n exy=%g\n exz=%g\n eyz=%g",
		&(para.L), &(para.t), &(para.V), &(para.es),&(para.stride),&(para.exy),&(para.exz),&(para.eyz)
		);
	}
	else
	{
		printf("\n[Error]:\n\tFailed to load parameters from parameters.txt.");
		printf("\n\tDefaulat values are restored.\n\n");
	}
	fclose(f);
	
	
	
	return para;
}

int checkrecord(char * s)
{
	if(s[0]=='#')
		return -1;
	if(s[0]=='\r')
		return -1;
	if(s[0]=='\n')
		return -1;

	return 1;	
}

gsl_matrix * load_klistone()
{
	FILE * f = fopen("klist.txt", "r");
	
	float k1;
	char s[100];
	int size=0;

	gsl_matrix * ks;
	
	if(f!=NULL)
	{	
		while((fgets(s, 100,f))!=NULL)	
		{
			if((checkrecord(s))>0)
				size++;
		}
		fclose(f);
		
			
		if(size<1)
		{
			return NULL;
		}			
		
		f = fopen("klist.txt", "r");
		
		if(f==NULL)
		{
			printf("Failed to load k list.\n");
			return NULL;
		}
		
		ks=gsl_matrix_alloc(size,1);
		size=0;
		k1=0;
		
		while((fgets(s, 100,f))!=NULL)	
		{	
			
			if((checkrecord(s))>0)
			{	
				//k1++;		
				sscanf(s, "%f", &k1);
				
				gsl_matrix_set(ks,size,0,k1);
				
				size++; 
				
			}
		}
		
	}
    else
    {
    	printf("Failed to load k list.\n");
    	return NULL;
    
	}


//	printf("\n%d sets of k loaded.\n",size);	
	fclose(f); 	
	
	return ks;
}



gsl_matrix * load_klist()
{
	FILE * f = fopen("klist.txt", "r");
	
	float k1,k2,k3;
	char s[100];
	int size=0;

	gsl_matrix * ks;
	
	if(f!=NULL)
	{	
		while((fgets(s, 100,f))!=NULL)	
		{
			if((checkrecord(s))>0)
				size++;
		}
		fclose(f);
		
			
		if(size<1)
		{
			return NULL;
		}			
		
		f = fopen("klist.txt", "r");
		
		if(f==NULL)
		{
			printf("Failed to load k list.\n");
			return NULL;
		}
		
		ks=gsl_matrix_alloc(size,3);
		size=0;
		k1=0;
		k2=0;
		k3=0;
		
		while((fgets(s, 100,f))!=NULL)	
		{	
			
			if((checkrecord(s))>0)
			{	
				//k1++;		
				sscanf(s, "%f %f %f", &k1, &k2,&k3);
				
				gsl_matrix_set(ks,size,0,k1);
				gsl_matrix_set(ks,size,1,k2);
				gsl_matrix_set(ks,size,2,k3);
				size++; 
				
			}
		}
		
	}
    else
    {
    	printf("Failed to load k list.\n");
    	return NULL;
    
	}


//	printf("\n%d sets of k loaded.\n",size);	
	fclose(f); 	
	
	return ks;
}

void load_sigma(gsl_matrix_complex * g1,
				gsl_matrix_complex * g2,
				gsl_matrix_complex * g3,
				gsl_matrix_complex * g4)
{
	gsl_matrix_complex_set_zero(g1);
	gsl_matrix_complex_set_zero(g2);
	gsl_matrix_complex_set_zero(g3);
	gsl_matrix_complex_set_zero(g4);
	
    gsl_complex i = gsl_complex_rect(0,1);        //  i
    gsl_complex ni = gsl_complex_rect(0,-1);      //  -i
    gsl_complex one = gsl_complex_rect(1,0);        //  1
    gsl_complex minus_one = gsl_complex_rect(-1,0);      //  -1
    
 //sigma x   
	gsl_matrix_complex_set(g1,0,1,one);
	gsl_matrix_complex_set(g1,1,0,one);

	
//sigma y
	gsl_matrix_complex_set(g2,0,1,ni);
	gsl_matrix_complex_set(g2,1,0,i);

	

//sigma z
	gsl_matrix_complex_set(g3,0,0,one);
	gsl_matrix_complex_set(g3,1,1,minus_one);
	
//Id 2x2
	gsl_matrix_complex_set(g4,0,0,one);
	gsl_matrix_complex_set(g4,1,1,one);	
}

void load_gammaold(gsl_matrix_complex * g1,
				gsl_matrix_complex * g2,
				gsl_matrix_complex * g3,
				gsl_matrix_complex * g4,
				gsl_matrix_complex * g5)
{
	gsl_matrix_complex_set_zero(g1);
	gsl_matrix_complex_set_zero(g2);
	gsl_matrix_complex_set_zero(g3);
	gsl_matrix_complex_set_zero(g4);
	gsl_matrix_complex_set_zero(g5);
    
    gsl_complex i = gsl_complex_rect(0,1);        //  i
    gsl_complex ni = gsl_complex_rect(0,-1);      //  -i
    gsl_complex one = gsl_complex_rect(1,0);        //  1
    gsl_complex minus_one = gsl_complex_rect(-1,0);      //  -1
    
    // g1
    //  0  0  0  I
    //  0  0  I  0
    //  0 -I  0  0
    // -I  0  0  0    
	gsl_matrix_complex_set(g1,0,3,i);
	gsl_matrix_complex_set(g1,1,2,i);
	gsl_matrix_complex_set(g1,2,1,ni);
	gsl_matrix_complex_set(g1,3,0,ni);
	
	// g2
    //  0  0  0  1
    //  0  0 -1  0
    //  0 -1  0  0
    //  1  0  0  0    
	gsl_matrix_complex_set(g2,0,3,one);
	gsl_matrix_complex_set(g2,1,2,minus_one);
	gsl_matrix_complex_set(g2,2,1,minus_one);
	gsl_matrix_complex_set(g2,3,0,one);
	

    // g3
    //  0  0  I  0
    //  0  0  0 -I
    // -I  0  0  0
    //  0  I  0  0    
	gsl_matrix_complex_set(g3,0,2,i);
	gsl_matrix_complex_set(g3,1,3,ni);
	gsl_matrix_complex_set(g3,2,0,ni);
	gsl_matrix_complex_set(g3,3,1,i);
	
	// g4
    //  0  0  1  0
    //  0  0  0  1
    //  1  0  0  0
    //  0  1  0  0    
	gsl_matrix_complex_set(g4,0,2,one);
	gsl_matrix_complex_set(g4,1,3,one);
	gsl_matrix_complex_set(g4,2,0,one);
	gsl_matrix_complex_set(g4,3,1,one);
	
	// g5
    //  1  0  0  0
    //  0  1  0  0
    //  0  0 -1  0
    //  0  0  0 -1    
	gsl_matrix_complex_set(g5,0,0,one);
	gsl_matrix_complex_set(g5,1,1,one);
	gsl_matrix_complex_set(g5,2,2,minus_one);
	gsl_matrix_complex_set(g5,3,3,minus_one);
	
}

//------------------------------------------------------------------------------
//	add a to the element of m(i,j)
//------------------------------------------------------------------------------
void matrix_add_at(gsl_matrix * m, int i, int j, double a)
{
	double t = gsl_matrix_get(m, i, j);
	gsl_matrix_set(m,i,j, t+a);
}

void cmatrix_add_at(gsl_matrix_complex * m, int i, int j, gsl_complex a)
{
	gsl_complex t = gsl_matrix_complex_get(m, i, j);
	t = gsl_complex_add(t,a);
	gsl_matrix_complex_set(m,i,j, t);
}

//------------------------------------------------------------------------------
//	print real matrix
//------------------------------------------------------------------------------
void print_mat(gsl_matrix * data, int const n1, int const n2)
{
	int i, j;
		
	for(i=0; i<n1;i++)
	{
		printf("\n");
	 	for(j=0;j<n2;j++)
	 	{
	
	   	 	printf("%g   ", gsl_matrix_get(data,i,j));
	 	}
	}
}

//------------------------------------------------------------------------------
//	print real matrix to file
//------------------------------------------------------------------------------
void fprint_mat(gsl_matrix * data, int const n1, int const n2, FILE * f)
{
	int i, j;
		
	for(i=0; i<n1;i++)
	{
	 	fprintf(f,"\n");	 			
	 	for(j=0;j<n2;j++)
	 	{	
	   	 	fprintf(f,"%g   ", gsl_matrix_get(data,i,j));
	 	}	 	

	}
}

//------------------------------------------------------------------------------
//	print complex matrix
//------------------------------------------------------------------------------
void print_cmat(gsl_matrix_complex * data, int const n1, int const n2)
{
	int i, j;
	gsl_complex c_temp1;
	printf("\n------------------");
	for(i=0; i<n1;i++)
	{
		printf("\n");
	 	for(j=0;j<n2;j++)
	 	{
	 		c_temp1 = gsl_matrix_complex_get(data,i,j);
	   	 	printf("%g + I %g    \t", GSL_REAL(c_temp1), GSL_IMAG(c_temp1));
	 	}
	}
	printf("\n------------------\n");
}

//------------------------------------------------------------------------------
//	print complex matrix to file
//------------------------------------------------------------------------------
void fprint_cmat(gsl_matrix_complex * data, int const n1, int const n2, FILE *f)
{
	int i, j;
	gsl_complex c_temp1;
	
	for(i=0; i<n1;i++)
	{
	 	fprintf(f,"\n");	 			
	 	for(j=0;j<n2;j++)
	 	{
	 		c_temp1 = gsl_matrix_complex_get(data,i,j);
	   	 	fprintf(f, "%g + I %g  ", GSL_REAL(c_temp1), GSL_IMAG(c_temp1));
	 	}

	}
}

//------------------------------------------------------------------------------
//	print complex matrix to file
//------------------------------------------------------------------------------
void fprint_cmat2(gsl_matrix_complex * data, int const n1, int const n2, FILE *fre, FILE *fimg)
{
	int i, j;
	gsl_complex c_temp1;
	
	for(i=0; i<n1;i++)
	{
	 	fprintf(fre,"\n");
	 	fprintf(fimg,"\n");	 			
	 	for(j=0;j<n2;j++)
	 	{
	 		c_temp1 = gsl_matrix_complex_get(data,i,j);
	   	 	fprintf(fre, "%g ", GSL_REAL(c_temp1));
	   	 	fprintf(fimg, "%g ", GSL_IMAG(c_temp1));
	 	}

	}
}


void gsl_matrix_complex_add_at(gsl_matrix_complex * g, int const i, int const j, gsl_complex a)
{
	gsl_complex temp;
	
	temp = gsl_matrix_complex_get(g,i,j);
	temp = gsl_complex_add(temp, a);
	gsl_matrix_complex_set(g,i,j,temp);        
	
}

void fprint_vec(gsl_vector * data, int const n1,  FILE * f)
{
  int i;
  for(i=0;i<n1;i++)
    	fprintf(f, "%g  ", gsl_vector_get(data,i));
}

void check_hermitian(gsl_matrix_complex * data, int const n)
{
	
	int i, j;
	gsl_complex t1, t2;
	
	for(i=0;i<n;i++)
		for(j=0;j<i;j++)
		{
			t1=gsl_matrix_complex_get(data,i,j);
			t2=gsl_matrix_complex_get(data,j,i);
			t2=gsl_complex_conjugate(t2);
		    if(gsl_complex_abs(gsl_complex_sub(t1,t2))>1e-5)
		    {
    	    	printf("\n H not Hermitian. H(%d, %d)=%g+%g*i \t H(%d, %d)=%g+%g*i\n",i,j,GSL_REAL(t1),GSL_IMAG(t1),j,i,GSL_REAL(t2),GSL_IMAG(t2));
		    	return;
		    }
		}
		
	
}


void load_gamma(gsl_matrix_complex * g1,
				gsl_matrix_complex * g2,
				gsl_matrix_complex * g3,
				gsl_matrix_complex * g4,
				gsl_matrix_complex * g5)
{
	gsl_matrix_complex_set_zero(g1);
	gsl_matrix_complex_set_zero(g2);
	gsl_matrix_complex_set_zero(g3);
	gsl_matrix_complex_set_zero(g4);
	gsl_matrix_complex_set_zero(g5);
    
    gsl_complex i = gsl_complex_rect(0,1);        //  i
    gsl_complex ni = gsl_complex_rect(0,-1);      //  -i
    gsl_complex one = gsl_complex_rect(1,0);        //  1
    gsl_complex minus_one = gsl_complex_rect(-1,0);      //  -1
    
    // g1
    //  0  -I  0  0
    //  I  0   0  0
    //  0  0   0  I
    //  0  0  -I  0    
	gsl_matrix_complex_set(g1,0,1,ni);
	gsl_matrix_complex_set(g1,1,0,i);
	gsl_matrix_complex_set(g1,2,3,i);
	gsl_matrix_complex_set(g1,3,2,ni);
	
	// g2
    //  0  1  0  0
    //  1  0  0  0
    //  0  0  0  -1
    //  0 0  -1  0    
	gsl_matrix_complex_set(g2,0,1,one);
	gsl_matrix_complex_set(g2,1,0,one);
	gsl_matrix_complex_set(g2,2,3,minus_one);
	gsl_matrix_complex_set(g2,3,2,minus_one);
	

    // g3
    //  0  0  -I  0
    //  0  0  0 -I
    //  I  0  0  0
    //  0  I  0  0    
	gsl_matrix_complex_set(g3,0,2,ni);
	gsl_matrix_complex_set(g3,1,3,ni);
	gsl_matrix_complex_set(g3,2,0,i);
	gsl_matrix_complex_set(g3,3,1,i);
	
	// g4
    //  0  0  1  0
    //  0  0  0  1
    //  1  0  0  0
    //  0  1  0  0    
	gsl_matrix_complex_set(g4,0,2,one);
	gsl_matrix_complex_set(g4,1,3,one);
	gsl_matrix_complex_set(g4,2,0,one);
	gsl_matrix_complex_set(g4,3,1,one);
	
	// g5
    //  1  0  0  0
    //  0  -1  0  0
    //  0  0 -1  0
    //  0  0  0 1    
	gsl_matrix_complex_set(g5,0,0,one);
	gsl_matrix_complex_set(g5,1,1,minus_one);
	gsl_matrix_complex_set(g5,2,2,minus_one);
	gsl_matrix_complex_set(g5,3,3,one);
	
}


void load_gammaij(gsl_matrix_complex * g12,
				gsl_matrix_complex * g13,
				gsl_matrix_complex * g14,
				gsl_matrix_complex * g15,
				gsl_matrix_complex * g23,gsl_matrix_complex * g24,
				gsl_matrix_complex * g25,
				gsl_matrix_complex * g34,
				gsl_matrix_complex * g35,
				gsl_matrix_complex * g45)

{

	gsl_matrix_complex_set_zero(g12);
	gsl_matrix_complex_set_zero(g13);
	gsl_matrix_complex_set_zero(g14);
	gsl_matrix_complex_set_zero(g15);
	gsl_matrix_complex_set_zero(g23);
	gsl_matrix_complex_set_zero(g24);
	gsl_matrix_complex_set_zero(g25);
	gsl_matrix_complex_set_zero(g34);
	gsl_matrix_complex_set_zero(g35);
	gsl_matrix_complex_set_zero(g45);
    
    gsl_complex i = gsl_complex_rect(0,1);        //  i
    gsl_complex ni = gsl_complex_rect(0,-1);      //  -i
    gsl_complex one = gsl_complex_rect(1,0);        //  1
    gsl_complex minus_one = gsl_complex_rect(-1,0);      //  -1
    
//g12

	gsl_matrix_complex_set(g12,0,0,minus_one);
	gsl_matrix_complex_set(g12,1,1,one);
	gsl_matrix_complex_set(g12,2,2,minus_one);
	gsl_matrix_complex_set(g12,3,3,one);

//g13
	gsl_matrix_complex_set(g13,0,3,i);
	gsl_matrix_complex_set(g13,1,2,ni);
	gsl_matrix_complex_set(g13,2,1,i);
	gsl_matrix_complex_set(g13,3,0,ni);
//g14
	gsl_matrix_complex_set(g14,0,3,minus_one);
	gsl_matrix_complex_set(g14,1,2,one);
	gsl_matrix_complex_set(g14,2,1,one);
	gsl_matrix_complex_set(g14,3,0,minus_one);

//g15
	gsl_matrix_complex_set(g15,0,1,one);
	gsl_matrix_complex_set(g15,1,0,one);
	gsl_matrix_complex_set(g15,2,3,one);
	gsl_matrix_complex_set(g15,3,2,one);

//g23
	gsl_matrix_complex_set(g23,0,3,minus_one);
	gsl_matrix_complex_set(g23,1,2,minus_one);
	gsl_matrix_complex_set(g23,2,1,minus_one);
	gsl_matrix_complex_set(g23,3,0,minus_one);
//g24
	gsl_matrix_complex_set(g24,0,3,ni);
	gsl_matrix_complex_set(g24,1,2,ni);
	gsl_matrix_complex_set(g24,2,1,i);
	gsl_matrix_complex_set(g24,3,0,i);
//g25
	gsl_matrix_complex_set(g25,0,1,i);
	gsl_matrix_complex_set(g25,1,0,ni);
	gsl_matrix_complex_set(g25,2,3,i);
	gsl_matrix_complex_set(g25,3,2,ni);

//g34
	gsl_matrix_complex_set(g34,0,0,minus_one);
	gsl_matrix_complex_set(g34,1,1,minus_one);
	gsl_matrix_complex_set(g34,2,2,one);
	gsl_matrix_complex_set(g34,3,3,one);
//g35
	gsl_matrix_complex_set(g35,0,2,one);
	gsl_matrix_complex_set(g35,1,3,minus_one);
	gsl_matrix_complex_set(g35,2,0,one);
	gsl_matrix_complex_set(g35,3,1,minus_one);
//g45
	gsl_matrix_complex_set(g45,0,2,i);
	gsl_matrix_complex_set(g45,1,3,ni);
	gsl_matrix_complex_set(g45,2,0,ni);
	gsl_matrix_complex_set(g45,3,1,i);



}

