#ifndef _UTIL_H_
#define _UTIL_H_

#endif //_UTIL_H_

#define PI 3.1415926536

#include <stdio.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_complex.h>
#include <gsl/gsl_complex_math.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_fft_complex.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_eigen.h>


typedef struct {
    int L;    		// size of the system
	float t;		// hopping constant
	float V;		// d*g coupling constant
	float es;       // chemical potential  
	int stride;     // hopping stride in d, stride=2, d=sin 2k
	float exy;
	float exz;
	float eyz;
  } phys_para ;
 
phys_para load_para(); 
gsl_matrix * load_klist();
int checkrecord(char * s);
gsl_matrix * load_klistone();
void load_gammaold(gsl_matrix_complex * g1,
				gsl_matrix_complex * g2,
				gsl_matrix_complex * g3,
				gsl_matrix_complex * g4,
				gsl_matrix_complex * g5);
void load_gamma(gsl_matrix_complex * g1,
				gsl_matrix_complex * g2,
				gsl_matrix_complex * g3,
				gsl_matrix_complex * g4,
				gsl_matrix_complex * g5);

void load_gammaij(gsl_matrix_complex * g12,
				gsl_matrix_complex * g13,
				gsl_matrix_complex * g14,
				gsl_matrix_complex * g15,
				gsl_matrix_complex * g23,gsl_matrix_complex * g24,
				gsl_matrix_complex * g25,
				gsl_matrix_complex * g34,
				gsl_matrix_complex * g35,
				gsl_matrix_complex * g45);
  
void load_sigma(gsl_matrix_complex * g1,
				gsl_matrix_complex * g2,
				gsl_matrix_complex * g3,
				gsl_matrix_complex * g4);

//------------------------------------------------------------------------------
//	add a to the element of m(i,j)
//------------------------------------------------------------------------------
void matrix_add_at(gsl_matrix * m, int i, int j, double a);  
void cmatrix_add_at(gsl_matrix_complex * m, int i, int j, gsl_complex a);

//------------------------------------------------------------------------------
//	print real matrix
//------------------------------------------------------------------------------
void print_mat(gsl_matrix * data, int const n1, int const n2);



//------------------------------------------------------------------------------
//	print real matrix to file
//------------------------------------------------------------------------------
void fprint_mat(gsl_matrix * data, int const n1, int const n2, FILE * f);


//------------------------------------------------------------------------------
//	print complex matrix
//------------------------------------------------------------------------------
void print_cmat(gsl_matrix_complex * data, int const n1, int const n2);


//------------------------------------------------------------------------------
//	print complex matrix to file
//------------------------------------------------------------------------------
void fprint_cmat(gsl_matrix_complex * data, int const n1,int const n2,FILE * f);
void fprint_cmat2(gsl_matrix_complex * data, int const n1, int const n2, FILE *fre, FILE *fimg);

void gsl_matrix_complex_add_at(gsl_matrix_complex * g, int const i, int const j, gsl_complex a);


void fprint_vec(gsl_vector * data, int const n1,  FILE * f);

void check_hermitian(gsl_matrix_complex * data, int const n);




